@@ -7,13 +7,13 @@ import java.sql.*;
 import java.util.*;
 import java.util.concurrent.Executor;
 import java.util.concurrent.atomic.AtomicInteger;
-import java.util.concurrent.locks.ReentrantLock;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import javax.sql.ConnectionEvent;
 import org.mariadb.jdbc.client.Client;
 import org.mariadb.jdbc.client.Context;
 import org.mariadb.jdbc.client.impl.StandardClient;
+import org.mariadb.jdbc.client.util.ClosableLock;
 import org.mariadb.jdbc.export.ExceptionFactory;
 import org.mariadb.jdbc.message.client.ChangeDbPacket;
 import org.mariadb.jdbc.message.client.PingPacket;
@@ -24,19 +24,23 @@ import org.mariadb.jdbc.util.constants.Capabilities;
 import org.mariadb.jdbc.util.constants.CatalogTerm;
 import org.mariadb.jdbc.util.constants.ConnectionState;
 import org.mariadb.jdbc.util.constants.ServerStatus;
+import org.mariadb.jdbc.util.timeout.NoOpQueryTimeoutHandler;
+import org.mariadb.jdbc.util.timeout.QueryTimeoutHandler;
+import org.mariadb.jdbc.util.timeout.QueryTimeoutHandlerImpl;
 
 /** Public Connection class */
 public class Connection implements java.sql.Connection {
 
+  // Redundant character escape '\\}' in RegExp is expected for Android. see CONJ-1161
   private static final Pattern CALLABLE_STATEMENT_PATTERN =
       Pattern.compile(
           "^(\\s*\\{)?\\s*((\\?\\s*=)?(\\s*/\\*([^*]|\\*[^/])*\\*/)*\\s*"
-              + "call(\\s*/\\*([^*]|\\*[^/])*\\*/)*\\s*((((`[^`]+`)|([^`}]+))\\.)?"
-              + "((`[^`]+`)|([^`}(]+)))\\s*(\\(.*\\))?(\\s*/\\*([^*]|\\*[^/])*\\*/)*"
-              + "\\s*(#.*)?)\\s*(}\\s*)?$",
+              + "call(\\s*/\\*([^*]|\\*[^/])*\\*/)*\\s*((((`[^`]+`)|([^`\\}]+))\\.)?"
+              + "((`[^`]+`)|([^`\\}(]+)))\\s*(\\(.*\\))?(\\s*/\\*([^*]|\\*[^/])*\\*/)*"
+              + "\\s*(#.*)?)\\s*(\\}\\s*)?$",
           Pattern.CASE_INSENSITIVE | Pattern.DOTALL);
 
-  private final ReentrantLock lock;
+  private final ClosableLock lock;
   private final Configuration conf;
   private final Client client;
   private final Properties clientInfo = new Properties();
@@ -50,6 +54,7 @@ public class Connection implements java.sql.Connection {
   private int lowercaseTableNames = -1;
   private boolean readOnly;
   private MariaDbPoolConnection poolConnection;
+  private QueryTimeoutHandler queryTimeoutHandler;
 
   /**
    * Connection construction.
@@ -59,7 +64,7 @@ public class Connection implements java.sql.Connection {
    * @param client client object
    */
   @SuppressWarnings({"this-escape"})
-  public Connection(Configuration conf, ReentrantLock lock, Client client) {
+  public Connection(Configuration conf, ClosableLock lock, Client client) {
     this.conf = conf;
     this.forceTransactionEnd =
         Boolean.parseBoolean(conf.nonMappedOptions().getProperty("forceTransactionEnd", "false"));
@@ -69,7 +74,13 @@ public class Connection implements java.sql.Connection {
     Context context = this.client.getContext();
     this.canUseServerTimeout =
         context.getVersion().isMariaDBServer()
-            && context.getVersion().versionGreaterOrEqual(10, 1, 2);
+            && context.getVersion().versionGreaterOrEqual(10, 1, 2)
+            && Boolean.parseBoolean(
+                conf.nonMappedOptions().getProperty("canUseServerTimeout", "true"));
+    this.queryTimeoutHandler =
+        this.canUseServerTimeout
+            ? NoOpQueryTimeoutHandler.INSTANCE
+            : new QueryTimeoutHandlerImpl(this, lock);
     this.canUseServerMaxRows =
         context.getVersion().isMariaDBServer()
             && context.getVersion().versionGreaterOrEqual(10, 3, 0);
@@ -102,7 +113,7 @@ public class Connection implements java.sql.Connection {
             : HostAddress.from(
                 currentIp, client.getHostAddress().port, client.getHostAddress().primary);
 
-    try (Client cli = new StandardClient(conf, hostAddress, new ReentrantLock(), true)) {
+    try (Client cli = new StandardClient(conf, hostAddress, new ClosableLock(), true)) {
       cli.execute(new QueryPacket("KILL QUERY " + client.getContext().getThreadId()), false);
     }
   }
@@ -196,46 +207,40 @@ public class Connection implements java.sql.Connection {
   }
 
   @Override
+  @SuppressWarnings("try")
   public void setAutoCommit(boolean autoCommit) throws SQLException {
     checkNotClosed();
     if (autoCommit == getAutoCommit()) {
       return;
     }
-    lock.lock();
-    try {
+    try (ClosableLock ignore = lock.closeableLock()) {
       getContext().addStateFlag(ConnectionState.STATE_AUTOCOMMIT);
       client.execute(
           new QueryPacket(((autoCommit) ? "set autocommit=1" : "set autocommit=0")), true);
-    } finally {
-      lock.unlock();
     }
   }
 
   @Override
+  @SuppressWarnings("try")
   public void commit() throws SQLException {
     checkNotClosed();
-    lock.lock();
-    try {
+    try (ClosableLock ignore = lock.closeableLock()) {
       if (forceTransactionEnd
           || (client.getContext().getServerStatus() & ServerStatus.IN_TRANSACTION) > 0) {
         client.execute(new QueryPacket("COMMIT"), false);
       }
-    } finally {
-      lock.unlock();
     }
   }
 
   @Override
+  @SuppressWarnings("try")
   public void rollback() throws SQLException {
     checkNotClosed();
-    lock.lock();
-    try {
+    try (ClosableLock ignore = lock.closeableLock()) {
       if (forceTransactionEnd
           || (client.getContext().getServerStatus() & ServerStatus.IN_TRANSACTION) > 0) {
         client.execute(new QueryPacket("ROLLBACK"), true);
       }
-    } finally {
-      lock.unlock();
     }
   }
 
@@ -295,17 +300,15 @@ public class Connection implements java.sql.Connection {
   }
 
   @Override
+  @SuppressWarnings("try")
   public void setReadOnly(boolean readOnly) throws SQLException {
     checkNotClosed();
-    lock.lock();
-    try {
+    try (ClosableLock ignore = lock.closeableLock()) {
       if (this.readOnly != readOnly) {
         client.setReadOnly(readOnly);
       }
       this.readOnly = readOnly;
       getContext().addStateFlag(ConnectionState.STATE_READ_ONLY);
-    } finally {
-      lock.unlock();
     }
   }
 
@@ -345,6 +348,7 @@ public class Connection implements java.sql.Connection {
     }
   }
 
+  @SuppressWarnings("try")
   private void setDatabase(String database) throws SQLException {
     checkNotClosed();
     // null catalog means keep current.
@@ -354,33 +358,30 @@ public class Connection implements java.sql.Connection {
             && database.equals(client.getContext().getDatabase()))) {
       return;
     }
-    lock.lock();
-    try {
+    try (ClosableLock ignore = lock.closeableLock()) {
       getContext().addStateFlag(ConnectionState.STATE_DATABASE);
       client.execute(new ChangeDbPacket(database), true);
       client.getContext().setDatabase(database);
-    } finally {
-      lock.unlock();
     }
   }
 
   @Override
   public int getTransactionIsolation() throws SQLException {
     checkNotClosed();
-    if (conf.useLocalSessionState() && client.getContext().getTransactionIsolationLevel() != null) {
+    boolean useContextState =
+        conf.useLocalSessionState()
+            || (client.getContext().hasClientCapability(Capabilities.CLIENT_SESSION_TRACK)
+                && ((client.getContext().getVersion().isMariaDBServer()
+                        && (client.getContext().getVersion().versionGreaterOrEqual(10, 2, 2)))
+                    || client.getContext().getVersion().versionGreaterOrEqual(5, 7, 0)));
+    if (useContextState && client.getContext().getTransactionIsolationLevel() != null) {
       return client.getContext().getTransactionIsolationLevel();
     }
 
-    String sql = "SELECT @@session.tx_isolation";
-
-    if (!client.getContext().getVersion().isMariaDBServer()) {
-      if ((client.getContext().getVersion().getMajorVersion() >= 8
-              && client.getContext().getVersion().versionGreaterOrEqual(8, 0, 3))
-          || (client.getContext().getVersion().getMajorVersion() < 8
-              && client.getContext().getVersion().versionGreaterOrEqual(5, 7, 20))) {
-        sql = "SELECT @@session.transaction_isolation";
-      }
-    }
+    String sql =
+        client.getContext().canUseTransactionIsolation()
+            ? "SELECT @@session.transaction_isolation"
+            : "SELECT @@session.tx_isolation";
 
     try (Statement stmt = createStatement()) {
       ResultSet rs = stmt.executeQuery(sql);
@@ -410,9 +411,17 @@ public class Connection implements java.sql.Connection {
   }
 
   @Override
+  @SuppressWarnings("try")
   public void setTransactionIsolation(int level) throws SQLException {
     checkNotClosed();
-    if (conf.useLocalSessionState()
+    boolean useContextState =
+        conf.useLocalSessionState()
+            || (client.getContext().hasClientCapability(Capabilities.CLIENT_SESSION_TRACK)
+                && ((client.getContext().getVersion().isMariaDBServer()
+                        && (client.getContext().getVersion().versionGreaterOrEqual(10, 2, 2)))
+                    || client.getContext().getVersion().versionGreaterOrEqual(5, 7, 0)));
+
+    if (useContextState
         && client.getContext().getTransactionIsolationLevel() != null
         && level == client.getContext().getTransactionIsolationLevel()) {
       return;
@@ -435,14 +444,11 @@ public class Connection implements java.sql.Connection {
       default:
         throw new SQLException("Unsupported transaction isolation level");
     }
-    lock.lock();
-    try {
+    try (ClosableLock ignore = lock.closeableLock()) {
       checkNotClosed();
       getContext().addStateFlag(ConnectionState.STATE_TRANSACTION_ISOLATION);
-      client.getContext().setTransactionIsolationLevel(level);
+      if (conf.useLocalSessionState()) client.getContext().setTransactionIsolationLevel(level);
       client.execute(new QueryPacket(query), true);
-    } finally {
-      lock.unlock();
     }
   }
 
@@ -592,10 +598,10 @@ public class Connection implements java.sql.Connection {
   }
 
   @Override
+  @SuppressWarnings("try")
   public void rollback(java.sql.Savepoint savepoint) throws SQLException {
     checkNotClosed();
-    lock.lock();
-    try {
+    try (ClosableLock ignore = lock.closeableLock()) {
       if ((client.getContext().getServerStatus() & ServerStatus.IN_TRANSACTION) > 0) {
         if (savepoint instanceof Connection.MariaDbSavepoint) {
           client.execute(
@@ -608,16 +614,14 @@ public class Connection implements java.sql.Connection {
           throw exceptionFactory.create("Unknown savepoint type");
         }
       }
-    } finally {
-      lock.unlock();
     }
   }
 
   @Override
+  @SuppressWarnings("try")
   public void releaseSavepoint(java.sql.Savepoint savepoint) throws SQLException {
     checkNotClosed();
-    lock.lock();
-    try {
+    try (ClosableLock ignore = lock.closeableLock()) {
       if ((client.getContext().getServerStatus() & ServerStatus.IN_TRANSACTION) > 0) {
         if (savepoint instanceof Connection.MariaDbSavepoint) {
           client.execute(
@@ -630,8 +634,6 @@ public class Connection implements java.sql.Connection {
           throw exceptionFactory.create("Unknown savepoint type");
         }
       }
-    } finally {
-      lock.unlock();
     }
   }
 
@@ -711,12 +713,12 @@ public class Connection implements java.sql.Connection {
   }
 
   @Override
+  @SuppressWarnings("try")
   public boolean isValid(int timeout) throws SQLException {
     if (timeout < 0) {
       throw exceptionFactory.create("the value supplied for timeout is negative");
     }
-    lock.lock();
-    try {
+    try (ClosableLock ignore = lock.closeableLock()) {
       client.execute(PingPacket.INSTANCE, true);
       return true;
     } catch (SQLException sqle) {
@@ -726,8 +728,6 @@ public class Connection implements java.sql.Connection {
         poolConnection.close();
       }
       return false;
-    } finally {
-      lock.unlock();
     }
   }
 
@@ -772,6 +772,7 @@ public class Connection implements java.sql.Connection {
   }
 
   @Override
+  @SuppressWarnings("try")
   public void setNetworkTimeout(Executor executor, int milliseconds) throws SQLException {
     if (this.isClosed()) {
       throw exceptionFactory.create(
@@ -783,11 +784,8 @@ public class Connection implements java.sql.Connection {
     }
     getContext().addStateFlag(ConnectionState.STATE_NETWORK_TIMEOUT);
 
-    lock.lock();
-    try {
+    try (ClosableLock ignore = lock.closeableLock()) {
       client.setSocketTimeout(milliseconds);
-    } finally {
-      lock.unlock();
     }
   }
 
@@ -912,6 +910,25 @@ public class Connection implements java.sql.Connection {
     return exceptionFactory;
   }
 
+  /**
+   * Return a QueryTimeoutHandler for old server that don't support Statement timeout.
+   *
+   * @param queryTimeout query timeout
+   * @return a query timeout handler
+   */
+  public QueryTimeoutHandler handleTimeout(int queryTimeout) {
+    return queryTimeoutHandler.create(queryTimeout);
+  }
+
+  /**
+   * Internal : retrieve internal ClosableLock
+   *
+   * @return ClosableLock
+   */
+  protected ClosableLock getLock() {
+    return lock;
+  }
+
   /**
    * for _TEST_ only
    *
