@@ -8,7 +8,6 @@ import java.util.*;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.Executor;
-import java.util.concurrent.locks.ReentrantLock;
 import org.mariadb.jdbc.Configuration;
 import org.mariadb.jdbc.HostAddress;
 import org.mariadb.jdbc.Statement;
@@ -16,6 +15,7 @@ import org.mariadb.jdbc.client.Client;
 import org.mariadb.jdbc.client.Completion;
 import org.mariadb.jdbc.client.Context;
 import org.mariadb.jdbc.client.context.RedoContext;
+import org.mariadb.jdbc.client.util.ClosableLock;
 import org.mariadb.jdbc.export.ExceptionFactory;
 import org.mariadb.jdbc.export.Prepare;
 import org.mariadb.jdbc.message.ClientMessage;
@@ -47,7 +47,7 @@ public class MultiPrimaryClient implements Client {
   protected final Configuration conf;
 
   /** thread locker */
-  protected final ReentrantLock lock;
+  protected final ClosableLock lock;
 
   /** is connections explicitly closed */
   protected boolean closed = false;
@@ -63,7 +63,7 @@ public class MultiPrimaryClient implements Client {
    * @throws SQLException if fail to connect
    */
   @SuppressWarnings({"this-escape"})
-  public MultiPrimaryClient(Configuration conf, ReentrantLock lock) throws SQLException {
+  public MultiPrimaryClient(Configuration conf, ClosableLock lock) throws SQLException {
     this.conf = conf;
     this.lock = lock;
     deniedListTimeout =
