@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 // Copyright (c) 2012-2014 Monty Program Ab
-// Copyright (c) 2015-2023 MariaDB Corporation Ab
+// Copyright (c) 2015-2024 MariaDB Corporation Ab
 package org.mariadb.jdbc.integration;
 
 import static org.junit.jupiter.api.Assertions.*;
@@ -8,10 +8,7 @@ import static org.junit.jupiter.api.Assertions.*;
 import java.sql.*;
 import java.sql.Connection;
 import java.sql.Statement;
-import java.util.Locale;
-import java.util.Properties;
-import java.util.Timer;
-import java.util.TimerTask;
+import java.util.*;
 import java.util.concurrent.Executor;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
@@ -58,11 +55,11 @@ public class ConnectionTest extends Common {
     assertThrowsContains(
         SQLException.class,
         () -> DriverManager.getConnection("jdbc:mariadb:///db"),
-        "hostname must be set to connect socket if not using local socket or pipe");
+        "host, pipe or local socket must be set to connect socket");
     assertThrowsContains(
         SQLException.class,
         () -> DriverManager.getConnection("jdbc:mariadb:///db?socketFactory=test"),
-        "hostname must be set to connect socket");
+        "host, pipe or local socket must be set to connect socket");
   }
 
   @Test
@@ -530,6 +527,8 @@ public class ConnectionTest extends Common {
     for (int level : levels) {
       connection.setTransactionIsolation(level);
       assertEquals(level, connection.getTransactionIsolation());
+      connection.setTransactionIsolation(level);
+      assertEquals(level, connection.getTransactionIsolation());
     }
     connection.close();
     assertThrows(
@@ -546,6 +545,57 @@ public class ConnectionTest extends Common {
     }
   }
 
+  @Test
+  public void confDefaultIsolationLevel() throws SQLException {
+    int[] levels =
+        new int[] {
+          java.sql.Connection.TRANSACTION_READ_UNCOMMITTED,
+          java.sql.Connection.TRANSACTION_READ_COMMITTED,
+          java.sql.Connection.TRANSACTION_SERIALIZABLE,
+          java.sql.Connection.TRANSACTION_REPEATABLE_READ
+        };
+    String[] levelStr =
+        new String[] {"READ-UNCOMMITTED", "READ-COMMITTED", "SERIALIZABLE", "REPEATABLE-READ"};
+    for (int i = 0; i < levels.length; i++) {
+      try (Connection connection = createCon("transactionIsolation=" + levelStr[i])) {
+        assertEquals(levels[i], connection.getTransactionIsolation());
+      }
+    }
+  }
+
+  @Test
+  public void readOnlyConnection() throws SQLException {
+    String url =
+        password == null || password.isEmpty()
+            ? String.format(
+                "jdbc:mariadb://address=(host=%s)(port=%s)(type=replica)/%s?user=%s%s",
+                hostname, port, database, user, defaultOther)
+            : String.format(
+                "jdbc:mariadb://address=(host=%s)(port=%s)(type=replica)/%s?user=%s&password=%s%s",
+                hostname, port, database, user, password, defaultOther);
+
+    try (Connection connection = DriverManager.getConnection(url)) {
+      Statement stmt = connection.createStatement();
+      boolean canUseTransactionReadOnly =
+          (isMariaDBServer()
+                  && sharedConn.getContext().getVersion().getMajorVersion() < 23
+                  && sharedConn.getContext().getVersion().versionGreaterOrEqual(11, 1, 1))
+              || (!sharedConn.getContext().getVersion().isMariaDBServer()
+                  && ((sharedConn.getContext().getVersion().getMajorVersion() >= 8
+                          && sharedConn.getContext().getVersion().versionGreaterOrEqual(8, 0, 3))
+                      || (sharedConn.getContext().getVersion().getMajorVersion() < 8
+                          && sharedConn
+                              .getContext()
+                              .getVersion()
+                              .versionGreaterOrEqual(5, 7, 20))));
+      ResultSet rs =
+          stmt.executeQuery(
+              "SELECT @@" + (canUseTransactionReadOnly ? "transaction_read_only" : "tx_read_only"));
+      assertTrue(rs.next());
+      assertTrue(rs.getBoolean(1));
+    }
+  }
+
   @Test
   public void savepointTest() throws SQLException {
     try (Connection con = createCon()) {
@@ -749,17 +799,37 @@ public class ConnectionTest extends Common {
         String.format(
             "jdbc:mariadb://%s:%s/%s?user=%s&password=%s&%s",
             hostname, testPort, database, pamUser, pamPwd, defaultOther);
-    try {
-      try (Connection connection =
-          DriverManager.getConnection(connStr + "&restrictedAuth=dialog")) {
-        // must have succeeded
-        connection.getCatalog();
-      }
-    } catch (SQLException e) {
-      System.err.println("fail with connectionString : " + connStr + "&restrictedAuth=dialog");
-      throw e;
-    }
+    if ("1".equals(System.getenv("CLEAR_TEXT"))) {
+      // mysql_clear_password is not permit if not using SSL
+      assertThrowsContains(
+          SQLException.class,
+          () ->
+              DriverManager.getConnection(connStr + "&restrictedAuth=dialog,mysql_clear_password"),
+          "Cannot use authentication plugin mysql_clear_password if SSL is not enabled");
+    } else {
+      try {
+        try (Connection connection =
+            DriverManager.getConnection(connStr + "&restrictedAuth=dialog,mysql_clear_password")) {
+          // must have succeeded
+          connection.getCatalog();
+        }
+      } catch (SQLException e) {
 
+      }
+      try {
+        try (Connection connection =
+            DriverManager.getConnection(connStr + "&restrictedAuth=dialog,mysql_clear_password")) {
+          // must have succeeded
+          connection.getCatalog();
+        }
+      } catch (SQLException e) {
+        System.err.println(
+            "fail with connectionString : "
+                + connStr
+                + "&restrictedAuth=dialog,mysql_clear_password");
+        throw e;
+      }
+    }
     Common.assertThrowsContains(
         SQLException.class,
         () -> DriverManager.getConnection(connStr + "&restrictedAuth=other"),
@@ -800,10 +870,7 @@ public class ConnectionTest extends Common {
   public void windowsNamedPipe() throws SQLException {
     ResultSet rs = null;
     try {
-      rs =
-          sharedConn
-              .createStatement()
-              .executeQuery("select @@named_pipe,@@socket,@@named_pipe_full_access_group");
+      rs = sharedConn.createStatement().executeQuery("select @@named_pipe,@@socket");
     } catch (SQLException sqle) {
       // on non Windows system, named_pipe doesn't exist.
     }
@@ -814,9 +881,16 @@ public class ConnectionTest extends Common {
       String namedPipeName = rs.getString(2);
       System.out.println("namedPipeName:" + namedPipeName);
       if (!isMariaDBServer() && minVersion(8, 0, 14)) {
-        String namedPipeFullAccess = rs.getString(3);
-        System.out.println("namedPipeFullAccess:" + namedPipeFullAccess);
-        Assumptions.assumeTrue(namedPipeFullAccess != null && !namedPipeFullAccess.isEmpty());
+        try {
+          rs = sharedConn.createStatement().executeQuery("select @@named_pipe_full_access_group");
+          if (rs != null) {
+            String namedPipeFullAccess = rs.getString(1);
+            System.out.println("namedPipeFullAccess:" + namedPipeFullAccess);
+            Assumptions.assumeTrue(namedPipeFullAccess != null && !namedPipeFullAccess.isEmpty());
+          }
+        } catch (SQLException sqle) {
+          // on non Windows system, named_pipe doesn't exist.
+        }
       }
 
       // skip test if no namedPipeName was obtained because then we do not use a socket connection
@@ -849,6 +923,20 @@ public class ConnectionTest extends Common {
           assertTrue(rs2.next());
         }
       }
+      // connection host format host name
+      try (java.sql.Connection connection =
+          DriverManager.getConnection(
+              String.format(
+                  "jdbc:mariadb://address=(pipe=%s)/%s?%s&tcpAbortiveClose&tcpKeepAlive",
+                  namedPipeName,
+                  sharedConn.getCatalog(),
+                  mDefUrl.substring(mDefUrl.indexOf("?user=") + 1)))) {
+        connection.setNetworkTimeout(null, 300);
+        java.sql.Statement stmt = connection.createStatement();
+        try (ResultSet rs2 = stmt.executeQuery("SELECT 1")) {
+          assertTrue(rs2.next());
+        }
+      }
     }
   }
 
@@ -935,17 +1023,26 @@ public class ConnectionTest extends Common {
     }
     stmt.execute("FLUSH PRIVILEGES");
 
-    String url =
-        String.format(
-            "jdbc:mariadb:///%s?user=testSocket&password=heyPassw!µ20§rd&localSocket=%s&tcpAbortiveClose&tcpKeepAlive",
-            sharedConn.getCatalog(), path);
-
-    try (java.sql.Connection connection = DriverManager.getConnection(url)) {
+    try (java.sql.Connection connection =
+        DriverManager.getConnection(
+            String.format(
+                "jdbc:mariadb:///%s?user=testSocket&password=heyPassw!µ20§rd&localSocket=%s&tcpAbortiveClose&tcpKeepAlive",
+                sharedConn.getCatalog(), path))) {
       connection.setNetworkTimeout(null, 300);
       rs = connection.createStatement().executeQuery("select 1");
       assertTrue(rs.next());
     }
 
+    // host format
+    try (java.sql.Connection connection =
+        DriverManager.getConnection(
+            String.format(
+                "jdbc:mariadb://address=(localSocket=%s)/%s?user=testSocket&password=heyPassw!µ20§rd&tcpAbortiveClose&tcpKeepAlive",
+                path, sharedConn.getCatalog()))) {
+      connection.setNetworkTimeout(null, 300);
+      rs = connection.createStatement().executeQuery("select 1");
+      assertTrue(rs.next());
+    }
     Common.assertThrowsContains(
         SQLException.class,
         () ->
@@ -953,8 +1050,15 @@ public class ConnectionTest extends Common {
                 "jdbc:mariadb:///"
                     + sharedConn.getCatalog()
                     + "?user=testSocket&password=heyPassw!µ20§rd&localSocket=/wrongPath"),
-        "Socket fail to connect to host");
-
+        "Socket fail to connect to address=(localSocket=/wrongPath)");
+    Common.assertThrowsContains(
+        SQLException.class,
+        () ->
+            DriverManager.getConnection(
+                "jdbc:mariadb://address=(localSocket=/wrongPath)/"
+                    + sharedConn.getCatalog()
+                    + "?user=testSocket&password=heyPassw!µ20§rd"),
+        "Socket fail to connect to address=(localSocket=/wrongPath)");
     if (haveSsl()) {
       String serverCertPath = SslTest.retrieveCertificatePath();
       if (serverCertPath != null) {
@@ -975,6 +1079,8 @@ public class ConnectionTest extends Common {
     stmt.execute("DROP USER testSocket");
   }
 
+  public static int staticTestValue = 0;
+
   @Test
   public void socketFactoryTest() throws SQLException {
     try (Connection conn = createCon("socketFactory=" + SocketFactoryBasicTest.class.getName())) {
@@ -989,6 +1095,13 @@ public class ConnectionTest extends Common {
         SQLNonTransientConnectionException.class,
         () -> createCon("socketFactory=wrongClass"),
         "Socket factory failed to initialized with option \"socketFactory\" set to \"wrongClass\"");
+    assertEquals(0, staticTestValue);
+    Common.assertThrowsContains(
+        SQLNonTransientConnectionException.class,
+        () -> createCon("socketFactory=org.mariadb.jdbc.integration.util.WrongSocketFactoryTest"),
+        "Socket factory failed to initialized with option \"socketFactory\" set to"
+            + " \"org.mariadb.jdbc.integration.util.WrongSocketFactoryTest\"");
+    assertEquals(0, staticTestValue);
   }
 
   @Test
@@ -1002,6 +1115,7 @@ public class ConnectionTest extends Common {
   public void sslNotSet() throws SQLException {
     Assumptions.assumeTrue(
         !"skysql".equals(System.getenv("srv")) && !"skysql-ha".equals(System.getenv("srv")));
+    Assumptions.assumeFalse(!isMariaDBServer() && minVersion(8, 4, 0));
     Assumptions.assumeFalse(haveSsl());
     Common.assertThrowsContains(
         SQLException.class, () -> createCon("sslMode=trust"), "ssl not enabled in the server");
@@ -1066,7 +1180,7 @@ public class ConnectionTest extends Common {
     // ensure connecting without DB
     String connStr =
         String.format(
-            "jdbc:mariadb://%s:%s/?user=%s&password=%s&%s&createDatabaseIfNotExist",
+            "jdbc:mariadb://%s:%s/?user=%s&password=%s&%s&createDatabaseIfNotExist&allowPublicKeyRetrieval=true",
             hostname, port, user, password, defaultOther);
     try (Connection con = DriverManager.getConnection(connStr)) {
       con.createStatement().executeQuery("SELECT 1");
@@ -1075,7 +1189,7 @@ public class ConnectionTest extends Common {
     String nonExistentDatabase = "bla`f`l";
     connStr =
         String.format(
-            "jdbc:mariadb://%s:%s/%s?user=%s&password=%s&%s&createDatabaseIfNotExist",
+            "jdbc:mariadb://%s:%s/%s?user=%s&password=%s&%s&createDatabaseIfNotExist&allowPublicKeyRetrieval=true",
             hostname, port, nonExistentDatabase, user, password, defaultOther);
     try (Connection con = DriverManager.getConnection(connStr)) {
       ResultSet rs = con.createStatement().executeQuery("select DATABASE()");
@@ -1086,7 +1200,7 @@ public class ConnectionTest extends Common {
     nonExistentDatabase = "bla`f`l0";
     connStr =
         String.format(
-            "jdbc:mariadb:replication://%s:%s,%s:%s/%s?user=%s&password=%s&%s&createDatabaseIfNotExist",
+            "jdbc:mariadb:replication://%s:%s,%s:%s/%s?user=%s&password=%s&%s&createDatabaseIfNotExist&allowPublicKeyRetrieval=true",
             hostname, port, hostname, port, nonExistentDatabase, user, password, defaultOther);
     try (Connection con = DriverManager.getConnection(connStr)) {
       ResultSet rs = con.createStatement().executeQuery("select DATABASE()");
@@ -1257,4 +1371,25 @@ public class ConnectionTest extends Common {
       }
     }
   }
+
+  @Test
+  public void isClosed() throws SQLException {
+    Connection con = DriverManager.getConnection(mDefUrl);
+    Statement stmt = con.createStatement();
+    PreparedStatement preparedStatement = con.prepareStatement("SELECT ?");
+    assertFalse(stmt.isClosed());
+    assertFalse(preparedStatement.isClosed());
+
+    stmt.execute("SELECT 1");
+    preparedStatement.setInt(1, 1);
+    preparedStatement.execute();
+
+    assertFalse(stmt.isClosed());
+    assertFalse(preparedStatement.isClosed());
+
+    con.close();
+
+    assertTrue(stmt.isClosed());
+    assertTrue(preparedStatement.isClosed());
+  }
 }
