@@ -1,7 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 // Copyright (c) 2012-2014 Monty Program Ab
-// Copyright (c) 2015-2021 MariaDB Corporation Ab
-
+// Copyright (c) 2015-2024 MariaDB Corporation Ab
 package org.mariadb.jdbc.integration;
 
 import java.sql.*;
@@ -12,7 +11,7 @@ public class GssapiAuthenticationTest extends Common {
 
   @Test
   public void nativePassword() throws Exception {
-    Assumptions.assumeTrue(isWindows());
+    Assumptions.assumeTrue(isWindows() && isMariaDBServer());
     Statement stmt = sharedConn.createStatement();
     try {
       stmt.execute("INSTALL SONAME 'auth_gssapi'");
